% function MultiSeg_Th_ROF_main
%--------------------------------------------------------------------------
%   Multiphase segmentation method by ROF-thresholding
%
%   Usage: MultiSeg_Th_ROF_main;
% 
%   Inputs: 
%       - none;
%
%   Outputs: 
%       - none;
% 
% 
%   The original algorithm was proposed in the following paper, if you use 
%   this file or package for your work, please refer to the following
%   paper:
% 
%   [1]  X. Cai and G. Steidl, Multiclass Segmentation by Iterated ROF 
%        Thresholding, EMMCVPR(2013), Lecture Notes in Computer Science, 
%        Vol. 8081, pp. 237--250, Springer, 2013.
%
%   [2]  X. Cai, R. Chan, C.-B. Schonlieb, G. Steidl, and T. Zeng. 
%        Linkage Between Piecewise Constant Mumford-Shah Model and 
%        Rudin-Osher-Fatemi Model and Its Virtue in Image Segmentation. 
%        SIAM Journal on Scientific Computing, 41(6):B1310?B1340, 2019.
%  
% 
% 
%   parameters:
%       - mu: the regularization parameter
%       - tolU: stopping criterion for computing ROF model
%       - tolTh: stopping criterion for computing our new model
%       - phNum: phase number
%       - iteMaxC: maximum itereation mumber 
% 
% 
%   All Rights Reserved  
%   Code by: Xiaohao Cai
%   Last updated: 01/05/2013 
%--------------------------------------------------------------------------

close all; clear all; clc;


cd data
%% Examples: load image and set parematers
%------- Example 1: test the threshold updated is useful
% % load shape_twophase_noise_miss; f=Img;
% % 
% % % set parameters
% % mu=1;phNum=2;
% % tolU=1e-5;tolTh=1e-2;
% % iteMaxC=10;
%-------

%------- Example 2: fourphase image with close intensities  
load shape_fourphase_noise1;
f=Img;%M=max(max(f));m=min(min(f));f=(f-m)/(M-m);

% set parameters
mu=4; phNum=4;
tolU=1e-5;tolTh=1e-2;
iteMaxC=10;
%-------


cd ..

tic
%% step 1: solve ROF model
tau=0.05; % algorithm parameter
u=chambolle_fbs(f,tau,tolU,1/mu);

%% step 2: select the thresholds
% choose initialization thresholds automatically using K-Means method
c=ThdKmeans(f,phNum);

% iteratively update the thresholds
for ic=1:iteMaxC
    c0=c;
    for i=1:phNum-1
        phase{i}=u>c(i);
    end

    uu{1}=1-phase{1};
    fm(1)=sum(sum(f.*uu{1}))/sum(sum(uu{1}));
    for i=1:phNum-2
        uu{i+1}=(phase{i}-phase{i+1})>0;
        fm(i+1)=sum(sum(f.*uu{i+1}))/sum(sum(uu{i+1}));
    end
    uu{phNum}=phase{phNum-1};
    fm(phNum)=sum(sum(f.*uu{phNum}))/sum(sum(uu{phNum}));
    
    for i=1:phNum-1
        c(i)=mean(fm(i:i+1));
    end
    
    error(ic)=sqrt(sum(sum((c-c0).^2)));
    disp(['Ite: ',num2str(ic),',   Error C: ', num2str(error(ic)), '!']);
    if error(ic) < tolU
        break;
    end
end


%% showing the results
[m,n]=size(f);
us=zeros(m,n);
for iPh=1:phNum
%     us=us+uu{iPh}*sum(sum(f.*uu{iPh}))/sum(sum(uu{iPh}));
    us=us+iPh*uu{iPh};
end

figure;
subplot(1,2,1),imshow(f,[]),title('given image')
subplot(1,2,2),imshow(us,[]),title(['mu =' num2str(mu)])

